@@ -1,21 +1,11 @@
 /*
- * Copyright (C) 2011 - David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 David Goulet <david.goulet@polymtl.ca>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <popt.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,11 +13,17 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <config.h>
 
 #include <common/mi-lttng.h>
 
 #include "../command.h"
+#include "version.h"
+
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-version.1.h>
+;
+#endif
 
 enum {
 	OPT_HELP = 1,
@@ -43,19 +39,6 @@ static struct poptOption long_options[] = {
 	{0, 0, 0, 0, 0, 0, 0}
 };
 
-/*
- * usage
- */
-static void usage(FILE *ofp)
-{
-	fprintf(ofp, "usage: lttng version [OPTIONS]\n");
-	fprintf(ofp, "\n");
-	fprintf(ofp, "Options:\n");
-	fprintf(ofp, "  -h, --help               Show this help\n");
-	fprintf(ofp, "      --list-options       Simple listing of options\n");
-	fprintf(ofp, "\n");
-}
-
 /*
  *  create_version
  */
@@ -149,13 +132,12 @@ int cmd_version(int argc, const char **argv)
 	while ((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_HELP:
-			usage(stdout);
+			SHOW_HELP();
 			goto end;
 		case OPT_LIST_OPTIONS:
 			list_cmd_options(stdout, long_options);
 			goto end;
 		default:
-			usage(stderr);
 			ret = CMD_UNDEFINED;
 			goto end;
 		}
@@ -167,8 +149,17 @@ int cmd_version(int argc, const char **argv)
 		MSG("lttng version " VERSION " - " VERSION_NAME "%s",
 			GIT_VERSION[0] == '\0' ? "" : " - " GIT_VERSION);
 		MSG("\n" VERSION_DESCRIPTION "\n");
-		MSG("Web site: http://lttng.org");
+		MSG("Web site: https://lttng.org");
 		MSG("\n%s", lttng_license);
+		if (EXTRA_VERSION_NAME[0] != '\0') {
+			MSG("\nExtra version name: " EXTRA_VERSION_NAME);
+		}
+		if (EXTRA_VERSION_DESCRIPTION[0] != '\0') {
+			MSG("\nExtra version description:\n\t" EXTRA_VERSION_DESCRIPTION);
+		}
+		if (EXTRA_VERSION_PATCHES[0] != '\0') {
+			MSG("\nExtra version patches:\n\t" EXTRA_VERSION_PATCHES);
+		}
 	}
 
 end:
