@@ -139,10 +139,19 @@ function conf_proc_count()
 # Bail out on failure
 function validate_lttng_modules_present ()
 {
+	# Check for loadable modules.
 	modprobe -n lttng-tracer 2>/dev/null
-	if [ $? -ne 0  ]; then
-		BAIL_OUT "LTTng modules not detected."
+	if [ $? -eq 0 ]; then
+		return 0
 	fi
+
+	# Check for builtin modules.
+	ls /proc/lttng > /dev/null 2>&1
+	if [ $? -eq 0 ]; then
+		return 0
+	fi
+
+	BAIL_OUT "LTTng modules not detected."
 }
 
 function enable_kernel_lttng_event
@@ -1170,8 +1179,16 @@ function enable_ust_lttng_event_filter()
 	local sess_name="$1"
 	local event_name="$2"
 	local filter="$3"
+	local channel_name=$4
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-event "$event_name" -s $sess_name -u --filter "$filter" 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	if [ -z $channel_name ]; then
+		# default channel if none specified
+		chan=""
+	else
+		chan="-c $channel_name"
+	fi
+
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-event $chan "$event_name" -s $sess_name -u --filter "$filter" 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ok $? "Enable event $event_name with filtering for session $sess_name"
 }
 
